@@ -11,7 +11,6 @@
 
 #include "gu_uuid.h"
 
-#include "gcs_test_utils.hpp"
 #include "gcs_memb_test.hpp" // must be included last
 
 using namespace gcs_test;
@@ -248,9 +247,10 @@ Suite *gcs_memb_suite(void)
     Suite *suite = suite_create("GCS membership changes");
 
     TCase *tcase = tcase_create("gcs_memb");
-    suite_add_tcase (suite, tcase);
-    tcase_add_test  (tcase, gcs_memb_test_465);
+    suite_add_tcase  (suite, tcase);
+    tcase_add_test   (tcase, gcs_memb_test_465);
     tcase_add_test  (tcase, gcs_memb_test_465E);
+    tcase_set_timeout(tcase, 30);
 
     tcase = tcase_create("membership_service");
     suite_add_tcase (suite, tcase);
